#! /usr/bin/python2

import string
import sys
import os
import xf86config

TOP_MOD_DIR = "/usr/lib64/xorg/modules"

# This will add an entry to ModulePath section,
# with previous entries untouched.
def addModulePath(files, newPathEntry):
	prevModPath = []
	# Check for existing ModulePath
	if (files.module != None):
		prevModPath = string.split(files.module, ",")
	# First, add the default module dirs. We add the dirs in
	# reversed order, and reverse the list at the end. 
	newModPath = [TOP_MOD_DIR] 
	newModPath.append(TOP_MOD_DIR + "/extensions")
	for i in range(len(prevModPath)):
		mp = prevModPath[i]
		# Remove trailing "/", in case the config file
		# has been hand-edited
		if mp[len(mp) - 1] == "/":
			mp = mp[:(len(mp) - 1)]
		# Add to new module path
		if not mp in newModPath:
			newModPath.append(mp)
	# Add new path entry
	if not newPathEntry in newModPath:
		newModPath.append(newPathEntry)
	# Reverse list
	newModPath.reverse()
	files.module = string.join(newModPath, ",") 	

#
# Removes a module from the Modules section.
#
def removeModule(modules, moduleName):
	i = 0
	for m in modules.load:
		if (m.name == moduleName):
			modules.load.remove(i)
		i += 1

# ------------ 
# Main section
# ------------

# Read config file
(xconfig, xconfigpath) = xf86config.readConfigFile()

if (len(sys.argv) > 1):
	print "Usage: nvidia-config-xfree86"
	print "(creates " + xconfigpath +")"
	print "(the original config file is saved under " + xconfigpath + ".nvidia.bak)"
	sys.exit(1)
else:
	output_file = xconfigpath
	if os.access(xconfigpath, os.F_OK):

		screen = xf86config.getPrimaryScreen(xconfig)
		device = xf86config.lookupDevice(xconfig, screen.device)
		device.driver = "nvidia"

		if not xconfig.files:
			xconfig.files = xf86config.XF86ConfFiles()
                addModulePath(xconfig.files, TOP_MOD_DIR + "/extensions/nvidia")

		if not xconfig.modules:
			xconfig.modules = xf86config.XF86ConfModule()
		removeModule(xconfig.modules, "dri")
		removeModule(xconfig.modules, "GLCore")

		# Write new XF86Config
		os.rename(xconfigpath, xconfigpath + ".nvidia.bak")
		xconfig.write(output_file)
	else:
		sys.exit(1)



